# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Dummy migration
        pass



    def backwards(self, orm):
    # Dummy migration
        pass


    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': (
                'django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [],
                     {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [],
                            {'to': "orm['auth.Permission']", 'symmetrical': 'False',
                             'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {
                'ordering': "('content_type__app_label', 'content_type__model', 'codename')",
                'unique_together': "(('content_type', 'codename'),)",
                'object_name': 'Permission'},
            'codename': (
                'django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [],
                             {'to': "orm['contenttypes.ContentType']"}),
            'id': (
                'django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [],
                            {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [],
                      {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [],
                           {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [],
                       {'to': "orm['auth.Group']", 'symmetrical': 'False',
                        'blank': 'True'}),
            'id': (
                'django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': (
                'django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': (
                'django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': (
                'django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [],
                           {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [],
                          {'max_length': '30', 'blank': 'True'}),
            'password': (
                'django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': (
                'django.db.models.fields.related.ManyToManyField', [],
                {'to': "orm['auth.Permission']", 'symmetrical': 'False',
                 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [],
                         {'unique': 'True', 'max_length': '30'})
        },
        'cms.cmsplugin': {
            'Meta': {'object_name': 'CMSPlugin'},
            'changed_date': ('django.db.models.fields.DateTimeField', [],
                             {'auto_now': 'True', 'blank': 'True'}),
            'creation_date': ('django.db.models.fields.DateTimeField', [],
                              {'default': 'datetime.datetime.now'}),
            'id': (
                'django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language': ('django.db.models.fields.CharField', [],
                         {'max_length': '15', 'db_index': 'True'}),
            'level': ('django.db.models.fields.PositiveIntegerField', [],
                      {'db_index': 'True'}),
            'lft': ('django.db.models.fields.PositiveIntegerField', [],
                    {'db_index': 'True'}),
            'parent': ('django.db.models.fields.related.ForeignKey', [],
                       {'to': "orm['cms.CMSPlugin']", 'null': 'True',
                        'blank': 'True'}),
            'placeholder': ('django.db.models.fields.related.ForeignKey', [],
                            {'to': "orm['cms.Placeholder']", 'null': 'True'}),
            'plugin_type': ('django.db.models.fields.CharField', [],
                            {'max_length': '50', 'db_index': 'True'}),
            'position': ('django.db.models.fields.PositiveSmallIntegerField', [],
                         {'null': 'True', 'blank': 'True'}),
            'rght': ('django.db.models.fields.PositiveIntegerField', [],
                     {'db_index': 'True'}),
            'tree_id': ('django.db.models.fields.PositiveIntegerField', [],
                        {'db_index': 'True'})
        },
        'cms.globalpagepermission': {
            'Meta': {'object_name': 'GlobalPagePermission'},
            'can_add': (
                'django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'can_change': (
                'django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'can_change_advanced_settings': (
                'django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'can_change_permissions': (
                'django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'can_delete': (
                'django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'can_moderate': (
                'django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'can_move_page': (
                'django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'can_publish': (
                'django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'can_recover_page': (
                'django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'can_view': (
                'django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'group': ('django.db.models.fields.related.ForeignKey', [],
                      {'to': "orm['auth.Group']", 'null': 'True', 'blank': 'True'}),
            'id': (
                'django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'sites': ('django.db.models.fields.related.ManyToManyField', [],
                      {'symmetrical': 'False', 'to': "orm['sites.Site']",
                       'null': 'True', 'blank': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [],
                     {'to': "orm['auth.User']", 'null': 'True', 'blank': 'True'})
        },
        'cms.page': {
            'Meta': {'ordering': "('site', 'tree_id', 'lft')",
                     'object_name': 'Page'},
            'changed_by': (
                'django.db.models.fields.CharField', [], {'max_length': '70'}),
            'changed_date': ('django.db.models.fields.DateTimeField', [],
                             {'auto_now': 'True', 'blank': 'True'}),
            'created_by': (
                'django.db.models.fields.CharField', [], {'max_length': '70'}),
            'creation_date': ('django.db.models.fields.DateTimeField', [],
                              {'auto_now_add': 'True', 'blank': 'True'}),
            'id': (
                'django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'in_navigation': ('django.db.models.fields.BooleanField', [],
                              {'default': 'True', 'db_index': 'True'}),
            'level': ('django.db.models.fields.PositiveIntegerField', [],
                      {'db_index': 'True'}),
            'lft': ('django.db.models.fields.PositiveIntegerField', [],
                    {'db_index': 'True'}),
            'limit_visibility_in_menu': (
                'django.db.models.fields.SmallIntegerField', [],
                {'default': 'None', 'null': 'True', 'db_index': 'True',
                 'blank': 'True'}),
            'login_required': (
                'django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'moderator_state': ('django.db.models.fields.SmallIntegerField', [],
                                {'default': '1', 'blank': 'True'}),
            'navigation_extenders': ('django.db.models.fields.CharField', [],
                                     {'db_index': 'True', 'max_length': '80',
                                      'null': 'True', 'blank': 'True'}),
            'parent': ('django.db.models.fields.related.ForeignKey', [],
                       {'blank': 'True', 'related_name': "'children'",
                        'null': 'True', 'to': "orm['cms.Page']"}),
            'placeholders': ('django.db.models.fields.related.ManyToManyField', [],
                             {'to': "orm['cms.Placeholder']",
                              'symmetrical': 'False'}),
            'publication_date': ('django.db.models.fields.DateTimeField', [],
                                 {'db_index': 'True', 'null': 'True',
                                  'blank': 'True'}),
            'publication_end_date': ('django.db.models.fields.DateTimeField', [],
                                     {'db_index': 'True', 'null': 'True',
                                      'blank': 'True'}),
            'published': (
                'django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'publisher_is_draft': ('django.db.models.fields.BooleanField', [],
                                   {'default': 'True', 'db_index': 'True'}),
            'publisher_public': (
                'django.db.models.fields.related.OneToOneField', [],
                {'related_name': "'publisher_draft'", 'unique': 'True', 'null': 'True',
                 'to': "orm['cms.Page']"}),
            'publisher_state': ('django.db.models.fields.SmallIntegerField', [],
                                {'default': '0', 'db_index': 'True'}),
            'reverse_id': ('django.db.models.fields.CharField', [],
                           {'db_index': 'True', 'max_length': '40', 'null': 'True',
                            'blank': 'True'}),
            'rght': ('django.db.models.fields.PositiveIntegerField', [],
                     {'db_index': 'True'}),
            'site': ('django.db.models.fields.related.ForeignKey', [],
                     {'to': "orm['sites.Site']"}),
            'soft_root': ('django.db.models.fields.BooleanField', [],
                          {'default': 'False', 'db_index': 'True'}),
            'template': (
                'django.db.models.fields.CharField', [], {'max_length': '100'}),
            'tree_id': ('django.db.models.fields.PositiveIntegerField', [],
                        {'db_index': 'True'})
        },
        'cms.pagemoderator': {
            'Meta': {'object_name': 'PageModerator'},
            'id': (
                'django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'moderate_children': (
                'django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'moderate_descendants': (
                'django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'moderate_page': (
                'django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'page': ('django.db.models.fields.related.ForeignKey', [],
                     {'to': "orm['cms.Page']"}),
            'user': ('django.db.models.fields.related.ForeignKey', [],
                     {'to': "orm['auth.User']"})
        },
        'cms.pagemoderatorstate': {
            'Meta': {'ordering': "('page', 'action', '-created')",
                     'object_name': 'PageModeratorState'},
            'action': ('django.db.models.fields.CharField', [],
                       {'max_length': '3', 'null': 'True', 'blank': 'True'}),
            'created': ('django.db.models.fields.DateTimeField', [],
                        {'auto_now_add': 'True', 'blank': 'True'}),
            'id': (
                'django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'message': ('django.db.models.fields.TextField', [],
                        {'default': "''", 'max_length': '1000', 'blank': 'True'}),
            'page': ('django.db.models.fields.related.ForeignKey', [],
                     {'to': "orm['cms.Page']"}),
            'user': ('django.db.models.fields.related.ForeignKey', [],
                     {'to': "orm['auth.User']", 'null': 'True'})
        },
        'cms.pagepermission': {
            'Meta': {'object_name': 'PagePermission'},
            'can_add': (
                'django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'can_change': (
                'django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'can_change_advanced_settings': (
                'django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'can_change_permissions': (
                'django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'can_delete': (
                'django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'can_moderate': (
                'django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'can_move_page': (
                'django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'can_publish': (
                'django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'can_view': (
                'django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'grant_on': (
                'django.db.models.fields.IntegerField', [], {'default': '5'}),
            'group': ('django.db.models.fields.related.ForeignKey', [],
                      {'to': "orm['auth.Group']", 'null': 'True', 'blank': 'True'}),
            'id': (
                'django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'page': ('django.db.models.fields.related.ForeignKey', [],
                     {'to': "orm['cms.Page']", 'null': 'True', 'blank': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [],
                     {'to': "orm['auth.User']", 'null': 'True', 'blank': 'True'})
        },
        'cms.pageuser': {
            'Meta': {'object_name': 'PageUser', '_ormbases': ['auth.User']},
            'created_by': ('django.db.models.fields.related.ForeignKey', [],
                           {'related_name': "'created_users'",
                            'to': "orm['auth.User']"}),
            'user_ptr': ('django.db.models.fields.related.OneToOneField', [],
                         {'to': "orm['auth.User']", 'unique': 'True',
                          'primary_key': 'True'})
        },
        'cms.pageusergroup': {
            'Meta': {'object_name': 'PageUserGroup', '_ormbases': ['auth.Group']},
            'created_by': ('django.db.models.fields.related.ForeignKey', [],
                           {'related_name': "'created_usergroups'",
                            'to': "orm['auth.User']"}),
            'group_ptr': ('django.db.models.fields.related.OneToOneField', [],
                          {'to': "orm['auth.Group']", 'unique': 'True',
                           'primary_key': 'True'})
        },
        'cms.placeholder': {
            'Meta': {'object_name': 'Placeholder'},
            'default_width': (
                'django.db.models.fields.PositiveSmallIntegerField', [],
                {'null': 'True'}),
            'id': (
                'django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'slot': ('django.db.models.fields.CharField', [],
                     {'max_length': '50', 'db_index': 'True'})
        },
        'cms.title': {
            'Meta': {'unique_together': "(('language', 'page'),)",
                     'object_name': 'Title'},
            'application_urls': ('django.db.models.fields.CharField', [],
                                 {'db_index': 'True', 'max_length': '200',
                                  'null': 'True', 'blank': 'True'}),
            'creation_date': ('django.db.models.fields.DateTimeField', [],
                              {'default': 'datetime.datetime.now'}),
            'has_url_overwrite': ('django.db.models.fields.BooleanField', [],
                                  {'default': 'False', 'db_index': 'True'}),
            'id': (
                'django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language': ('django.db.models.fields.CharField', [],
                         {'max_length': '15', 'db_index': 'True'}),
            'menu_title': ('django.db.models.fields.CharField', [],
                           {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'meta_description': ('django.db.models.fields.TextField', [],
                                 {'max_length': '255', 'null': 'True',
                                  'blank': 'True'}),
            'meta_keywords': ('django.db.models.fields.CharField', [],
                              {'max_length': '255', 'null': 'True',
                               'blank': 'True'}),
            'page': ('django.db.models.fields.related.ForeignKey', [],
                     {'related_name': "'title_set'", 'to': "orm['cms.Page']"}),
            'page_title': ('django.db.models.fields.CharField', [],
                           {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'path': ('django.db.models.fields.CharField', [],
                     {'max_length': '255', 'db_index': 'True'}),
            'redirect': ('django.db.models.fields.CharField', [],
                         {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'slug': (
                'django.db.models.fields.SlugField', [], {'max_length': '255'}),
            'title': (
                'django.db.models.fields.CharField', [], {'max_length': '255'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)",
                     'unique_together': "(('app_label', 'model'),)",
                     'object_name': 'ContentType',
                     'db_table': "'django_content_type'"},
            'app_label': (
                'django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': (
                'django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': (
                'django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'sites.site': {
            'Meta': {'ordering': "('domain',)", 'object_name': 'Site',
                     'db_table': "'django_site'"},
            'domain': (
                'django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': (
                'django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        }
    }

    complete_apps = ['cms']
