{% load i18n cms_admin %}

<div id="sitemap" class="cms-tree">
<ul class="header">
	<li>
		<div class="cont">
			<div class="col1">{% trans "title" %}</div>
			<div class="col2">
				<div class="col-actions">{% trans "actions" %}</div>
				<div class="col-navigation">{% trans "in navigation" %}</div>
				<div class="col-published">{% trans "published" %}</div>
				<div class="col-published">{% trans "start" %}</div>
				<div class="col-published">{% trans "end" %}</div>
				<div class="col-view-perms">{% trans "restricted" %}</div>
				<div class="col-lastchange">{% trans "last changes" %}</div>
				<div class="col-creator">{% trans "changed by" %}</div>
			</div>
		</div>
	</li>
</ul>
<div class="tree{% if not CMS_PERMISSION or has_add_permission %} root_allow_children{% endif %}">
	<ul class="tree-default">
		{% for page in cl.get_items %}{% show_admin_menu page %}{% endfor %}
	</ul>
</div>
</div>