{% extends "admin/cms/page/change_form.html" %}
{% load i18n %}

{% block breadcrumbs %}
<div class="breadcrumbs">
	<a href="../../../../../">{% trans "Home" %}</a> &rsaquo;
	<a href="../../../../">{{ app_label|capfirst|escape }}</a> &rsaquo;
	<a href="../../../">{{ opts.verbose_name_plural|capfirst }}</a> &rsaquo;
	<a href="../../">{{ original|truncatewords:"18" }}</a> &rsaquo;
	<a href="../">{% trans "History" %}</a> &rsaquo;
	{% blocktrans with opts.verbose_name as verbose_name %}Revert {{ verbose_name }}{% endblocktrans %}
</div>
{% endblock %}

{% block content %}
{% with 1 as is_popup %}{{ block.super }}{% endwith %}
{% endblock %}

{% block form_top %}
<p>{% blocktrans %}Press the save button below to revert to this version of the object.{% endblocktrans %}</p>
{{ block.super }}
{% endblock %}