<li class="{% if active %}cms_toolbar-item-navigation-active{% endif %}{% if disabled %} cms_toolbar-item-navigation-disabled{% endif %} {{ extra_classes|join:' ' }}">
	<a href="{{ url }}"><span>{{ name }}</span></a>
</li>