@charset "utf-8";
/*!
 * @copyright:	https://github.com/divio/django-cms
 */

//##################################################
// #IMPORT SETTINGS#
@import "partials/settings";

//##################################################
// #MODAL#
#content h2:first-child { display:none; }
#content { margin:10px 10px 0; }
#content .module { padding:10px; }

#container { padding-top:0; }

#footer { display:none; }

#cke_id_body {
	border:1px solid $color-grey-10 !important;

	.cke_top, .cke_bottom { background:$color-grey-5 !important; border-color:$color-grey-10 !important;
		@include box-shadow(none !important); }
}

.errornote { margin:4px 0 0 !important; }

#page_form_lang_tabs {
	.language_button { background:#ccc !important; }
	.language_button.selected { color:black !important; background:white !important; text-shadow:none !important; }
	.language_button.notfilled { color:#bbb !important; }
}
#content-main #lang_tab_content .header { display:block !important; background:white !important; margin-top:-1px; }
