{% load i18n %}
{% for cls in plugin_classes %}
	{% if forloop.first %}<li class="cms_dragmenu-subnav-title"><span>{% trans "Add Plugin" %}</span></li>{% endif %}
	<li><a data-rel="add" href="#{{ cls.0 }}">{{ cls.1 }}</a></li>
{% endfor %}