@charset "utf-8";/*
 * @copyright:	https://github.com/divio/django-cms
 */#header{display:none}#container{padding-top:30px}#container .breadcrumbs{display:block;background:#666 !important}#content{padding-top:10px}.change-form #content{margin-top:0}.submit-row{padding:10px 10px 5px !important}.submit-row input.default,.object-tools .addlink{font-weight:200 !important;padding:5px 12px !important;margin-bottom:5px !important;border:1px solid #0e72ec !important;background-image:-webkit-gradient(linear, 50% 0%, 50% 100%, color-stop(0%, #0e97ec), color-stop(100%, #0e72ec)) !important;background-image:-webkit-linear-gradient(top, #0e97ec,#0e72ec) !important;background-image:-moz-linear-gradient(top, #0e97ec,#0e72ec) !important;background-image:-o-linear-gradient(top, #0e97ec,#0e72ec) !important;background-image:linear-gradient(top, #0e97ec,#0e72ec) !important;-webkit-box-shadow:inset #6accf6 0px 1px 0px !important;-moz-box-shadow:inset #6accf6 0px 1px 0px !important;box-shadow:inset #6accf6 0px 1px 0px !important}.submit-row input.default:hover,.submit-row input.default:active,.submit-row input.default:focus,.object-tools .addlink:hover,.object-tools .addlink:active,.object-tools .addlink:focus{background:#0e72ec !important;-webkit-box-shadow:none !important;-moz-box-shadow:none !important;box-shadow:none !important}.submit-row input.default:active,.submit-row input.default:focus,.object-tools .addlink:active,.object-tools .addlink:focus{background:#0b5bbc !important}.object-tools .recoverlink,.submit-row input{font-weight:200 !important;padding:5px 12px !important;margin-bottom:5px !important;border:1px solid #e6e6e6 !important;-webkit-box-shadow:none !important;-moz-box-shadow:none !important;box-shadow:none !important}
