//##################################################
// #SETTINGS#

// #IMPORTS#
@import "compass/css3";
@import "compass/utilities";

// #COLORS#
$color-white: #fff;
$color-black: #000;

$color-grey: #666;
$color-blue: #0eaaec;
$color-green: #693;
$color-orange: #c93;
$color-red: #f00;
$color-window: #fcfcfc;

// #COLORS/auto#
$color-grey-5: lighten($color-grey, 58%); // fafafa
$color-grey-10: lighten($color-grey, 50%); // e5e5e5
$color-grey-20: lighten($color-grey, 25%); // 454545
$color-grey-70: darken($color-grey, 13%); // 454545
$color-grey-90: darken($color-grey, 20%); // 333333

$gradient-toolbar: linear-gradient(top, lighten($color-grey, 58%) 0%, lighten($color-grey, 59%) 50%, lighten($color-grey, 58%) 100%);
$gradient-dark: linear-gradient(top, $color-grey, darken($color-grey, 20%));
$gradient-blue: linear-gradient(top, adjust-hue($color-blue, 5deg), adjust-hue($color-blue, 15deg));
$gradient-green: linear-gradient(top, lighten($color-green, 10%) 0%, saturate($color-green, 20%) 50%, saturate($color-green, 30%) 100%);
$gradient-orange: linear-gradient(top, lighten($color-orange, 5%), $color-orange);
$gradient-red: linear-gradient(top, adjust-hue($color-red, 5deg), adjust-hue($color-red, 15deg));
$gradient-light: linear-gradient(top, lighten($color-grey-10, 5%), $color-grey-10);
