<li class="{% if active %}cms_toolbar-item-navigation-active{% endif %}{% if disabled %} cms_toolbar-item-navigation-disabled{% endif %} {{ extra_classes|join:' ' }}">
	<a href="{{ url }}" data-rel="modal"{% if close_on_url_change %} data-close-on-url="{{ close_on_url }}"{% endif %}{% if on_close %} data-on-close="{{ on_close }}"{% endif %}><span>{{ name }}</span></a>
</li>