{% load i18n sekizai_tags cms_tags %}{% load url from future %}
{% addtoblock "js" %}
<script type="text/javascript">
(function($) {
// CMS.$ will be passed for $
$(document).ready(function () {
	new CMS.PlaceholderItem('#cms_placeholder-generic-{{ current_page.pk }}', {
		'type': 'generic',
		'plugin_name': 'Page Title',
		'urls': {
			'edit_plugin': '{% url "admin:cms_page_edit_title" current_page.pk lang %}'
		}
	});
});
})(CMS.$);
</script>
{% endaddtoblock %}
<span id="cms_placeholder-generic-{{ current_page.pk }}" class="cms_placeholder-generic">{% page_attribute title %}</span>