{% load i18n %}<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head height="100%">
<title>{{ title }}</title>
<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
<meta http-equiv="Content-Language" content="en-us" />
<style type="text/css" media="screen">
	html,
	body,
	.grid {
		height: 100%;
	}
	
	.header {
		margin-bottom: 40px;
	}
	
	.content {
		padding: 28px;
		font-family: Lucida Grande;
		font-size: 12px;
	}
	
	.content h1 {
		font-size: 20px;
		font-weight: normal;
	}
	
	.content a,
	.content a:active,
	.content a:visited,
	.content a:hover {
		text-decoration: underline;
	}
	
	.data {
		line-height: 24px;
	}
</style>
</head>

<body topmargin="0" leftmargin="0" marginheight="0" marginwidth="0" height="100%">
	<table border="0" cellpadding="0" cellspacing="0" width="100%" height="100%" class="grid">
		<tr>
			<td width="100%" class="content" valign="top">
				<div class="header">
					<h1>{{ title }}</h1>
					{% block header %}{% endblock %}
				</div>
				{% block content %}{% endblock %}
			</td>
		</tr>
		<tr>
			<td class="content">
				<p>{% blocktrans %}Log in to administration <a href="{{ login_url }}">here</a>.{% endblocktrans %}</p>
				<p>Generated by <a href="https://www.django-cms.org">django cms</a> {% now "d.m.Y H:i:s" %}.</p>
			</td>
		</tr>
	</table>
</body>
</html>