@charset "utf-8";/*
 * @copyright:	https://github.com/divio/django-cms
 */#page_form_lang_tabs{position:relative}#page_form_lang_tabs input.language_button{background:#ccc}#page_form_lang_tabs input.selected{color:black;text-shadow:none;background:white}#page_form_lang_tabs input.notfilled{color:#bbb;background:none}#page_form_lang_tabs .lang_tabs_line{position:absolute;left:0;bottom:-5px;width:100%;height:5px;background:white}.cms_dialog{position:absolute;left:50%;top:50%;z-index:99999;width:500px;height:200px;padding:25px;margin:-100px 0 0 -275px;background:white;border:1px solid #ccc;-webkit-border-radius:5px;-moz-border-radius:5px;-ms-border-radius:5px;-o-border-radius:5px;border-radius:5px}.cms_dialog h1{padding:0;margin:0 0 15px}.cms_dialog form{padding:15px 0;margin:15px 0;border-top:1px solid #f3f3f3}
