<li class="{% if active %}cms_toolbar-item-navigation-active{% endif %}{% if disabled %} cms_toolbar-item-navigation-disabled{% endif %} {{ extra_classes|join:' ' }}">
	<a href="{{ action }}" data-rel="ajax" data-post="{{ data }}"{% if question %} data-text="{{ question }}"{% endif %}><span>{{ name }}</span></a>
</li>