# -*- coding: utf-8 -*-


from cms.tests.admin import *
from cms.tests.api import *
from cms.tests.apphooks import *
from cms.tests.docs import *
from cms.tests.forms import *
from cms.tests.i18n import *
from cms.tests.mail import *
from cms.tests.menu import *
from cms.tests.menu_utils import *
from cms.tests.multilingual import *
from cms.tests.navextender import *
from cms.tests.nonroot import *
from cms.tests.page import *
from cms.tests.permissions import *
from cms.tests.permmod import *
from cms.tests.placeholder import *
from cms.tests.plugins import *
from cms.tests.po import *
from cms.tests.publisher import *
from cms.tests.rendering import *
from cms.tests.reversion_tests import *
from cms.tests.security import *
from cms.tests.settings import *
from cms.tests.site import *
from cms.tests.stacks import *
from cms.tests.staticfiles import *
from cms.tests.templatetags import *
from cms.tests.toolbar import *
from cms.tests.urlutils import *
from cms.tests.views import *
from cms.tests.management import *
from cms.tests.menu_page_viewperm import *
from cms.tests.menu_page_viewperm_staff import *
from cms.tests.nested_plugins import *
from cms.tests.check import *
from cms.tests.frontend import *
from cms.tests.no_i18n import *
