{% load cms_admin i18n %}

<li id="page_{{page.pk}}" class="{% if cl.is_filtered %}leaf{% endif %}{% if has_move_page_permission %} moveable{% endif %}"{% if metadata %} mdata="{{ metadata }}{% endif %}" rel="{% ifequal page.level 0 %}topnode{% else %}node{% endifequal %}">
	{% include "admin/cms/page/tree/menu_item.html" %}
	{% with page.childrens as children %}
	{% if children %}
	<ul{% if page.last %} class="last"{% endif %}>
		{% if page.id in open_menu_trees %}
			{% for child in children %}
				{% show_admin_menu child %}
			{% endfor %}
		{% endif %}
	</ul>
	{% endif %}
	{% endwith %}
</li>