{% extends "admin/change_form.html" %}{% load i18n %}{% load url from future %}

{% block breadcrumbs %}
<div class="breadcrumbs">
	<a href="{% url 'admin:index' %}">{% trans 'Home' %}</a>
	&rsaquo; <a href="{% url 'admin:app_list' app_label=opts.app_label %}">{{ app_label|capfirst|escape }}</a>
	&rsaquo; {{ opts.verbose_name_plural|capfirst }}
	&rsaquo; {% if add %}{% trans 'Add' %} {{ opts.verbose_name }}{% else %}{{ original|truncatewords:"18" }}{% endif %}
</div>
{% endblock %}

{% block object-tools %}{% endblock %}

{% block submit_buttons_bottom %}
<div class="submit-row">
	<input type="submit" value="{% trans 'Save' %}" class="default" name="_save" {{ onclick_attrib }}/>
</div>
{% endblock %}