{% load i18n cms_admin %}
<table>
	<thead><tr>
		<th>{% trans "Action" %}</th>
		<th>{% trans "Created" %}</th>
		<th>{% trans "User" %}</th>
		<th>{% trans "Message" %}</th>
	</tr></thead>
	<tbody>
		{% for state in page.pagemoderatorstate_set.all %}<tr class="{% cycle 'row1' 'row2' %}">
				<td class="action">{{ state.get_action_display }}</td>
				<td class="created">{{ state.created }}</td>
				<td class="user">{{ state.user }}</td>
				<td class="message">{{ state.message }}</td>
			</tr>
		{% endfor %}
	</tbody>
</table>