{% extends "admin/cms/page/history/revision_header.html" %}
{% load i18n %}
{% load url from future %}

{% block extrahead %}
{{ block.super }}
<script type="text/javascript" src="{% url "admin:jsi18n" %}"></script>
{{ media }}
{% endblock %}

{% block breadcrumbs %}
<div class="breadcrumbs">
	<a href="../../../../">{% trans "Home" %}</a> &rsaquo;
	<a href="../../../">{{ app_label|capfirst|escape }}</a> &rsaquo;
	<a href="../../">{{ opts.verbose_name_plural|capfirst }}</a> &rsaquo;
	<a href="../">{% blocktrans with opts.verbose_name as verbose_name %}Recover deleted {{ verbose_name }}{% endblocktrans %}</a> &rsaquo;
	{{ title }}
</div>
{% endblock %}

{% block form_top %}
<p>{% blocktrans %}Press the save button below to recover this version of the object.{% endblocktrans %}</p>
{% endblock %}