{% load i18n %}
{% load url from future %}

<div class="submit-row" {% if is_popup %}style="overflow: auto;"{% endif %}>
	{% if show_save %}<input type="submit" value="{% trans 'Save' %}" class="default" name="_save" {{ onclick_attrib }}/>{% endif %}

	<p class="deletelink-box">
	{% if show_delete_link %}<a href="delete/" class="deletelink">{% trans "Delete" %}</a>{% endif %}
	{% if basic_info %}<a href="{% url 'admin:cms_page_change' object_id %}?language={{ language }}" class="default basic_info">{% trans "Basic Infos" %}</a>{% endif %}
	{% if advanced_settings %}<a href="{% url 'admin:cms_page_advanced' object_id %}?language={{ language }}" class="default advanced_settings">{% trans "Advanced Settings" %}</a>{% endif %}
	</p>

	{% if show_save_as_new %}<input type="submit" value="{% trans 'Save as new' %}" name="_saveasnew" {{ onclick_attrib }}/>{%endif%}
	{% if show_save_and_add_another %}<input type="submit" value="{% trans 'Save and add another' %}" name="_addanother" {{ onclick_attrib }} />{% endif %}
	{% if show_save_and_continue %}<input type="submit" value="{% trans 'Save and continue editing' %}" name="_continue" {{ onclick_attrib }}/>{% endif %}
</div>